

puts "loading sys.V1725.tcl"

variable registerlist

lappend registerlist [list NetVar.V1725_NumSamples  "-n V1725_NumSamples -t int"]
lappend registerlist [list NetVar.V1725_VMERetries  "-n V1725_VMERetries -t int"]
lappend registerlist [list NetVar.V1725_BERREnable  "-n V1725_BERREnable -t int"]
lappend registerlist [list NetVar.V1725_VMEBufSize  "-n V1725_VMEBufSize -t int"]

variable V1725_Base 160

proc Set_V1725_Options {} {
    variable Options
    variable V1725_Base

puts "Set_V1725_Options"

    load_Options

   if {![info exists Options(V1725_MaxEvtsPerBuf)]}         {set Options(V1725_MaxEvtsPerBuf) 1}
   if {![info exists Options(V1725_MaxBLT)]}                {set Options(V1725_MaxBLT) 1}
   if {![info exists Options(V1725_MemBufSize)]}            {set Options(V1725_MemBufSize) 1024}
   if {![info exists Options(V1725_VMEBufSize)]}            {set Options(V1725_VMEBufSize) 16384}

   if {![info exists Options(V1725_Hist_Trace.Enable)]}    {set Options(V1725_Hist_Trace.Enable) 1}
   if {![info exists Options(V1725_Hist_Trace.channels)]}  {set Options(V1725_Hist_Trace.channels) 512}
   if {![info exists Options(V1725_Hist_Trace.offset)]}    {set Options(V1725_Hist_Trace.offset) 0}
   if {![info exists Options(V1725_Hist_Trace.shift)]}     {set Options(V1725_Hist_Trace.shift) 0}

   if {![info exists Options(V1725_GroupBase)]}             {set Options(V1725_GroupBase) $V1725_Base}
   set V1725_Base  $Options(V1725_GroupBase)

    update_Options
}

proc V1725_Configure {} {
    variable Options

puts "V1725_Configure"

    set VME_BUFFER_SIZE [ReadNetVar NetVar.V1725_VMEBufSize]
    set Options(V1725_VMEBufSize) $VME_BUFFER_SIZE

      update_Options

           switch $Options(V1725_MemBufSize) {
              65536 {set BUFF_ORG 2; set BUFF_NUM 4}
              32768 {set BUFF_ORG 3; set BUFF_NUM 8}
              16384 {set BUFF_ORG 4; set BUFF_NUM 16}
              8192  {set BUFF_ORG 5; set BUFF_NUM 32}
              4096  {set BUFF_ORG 6; set BUFF_NUM 64}
              2048  {set BUFF_ORG 7; set BUFF_NUM 128}
              1024  {set BUFF_ORG 8; set BUFF_NUM 256}
              512   {set BUFF_ORG 9; set BUFF_NUM 512}
              256   {set BUFF_ORG 10; set BUFF_NUM 1024}
              default {set BUFF_ORG 10; set BUFF_NUM 1024}
           }

      set z [catch {ReadVMERegisterAttr v1725#1.VMEStatus} m]
      if {$z==0} then {

           set z [catch {WriteVMERegister v1725#1.VMEControl 16} m];  #  force BERR option

           set z [catch {ReadVMERegister v1725#1.CustomSize} SIZE];
           if {$SIZE <= 0} {set SIZE 32}
           WriteNetVar NetVar.V1725_NumSamples [expr $SIZE*10]

           set NUM [expr (($Options(V1725_MemBufSize) - 2) * 2) / $SIZE]
           if {$NUM > $Options(V1725_MaxEvtsPerBuf)} {set NUM $Options(V1725_MaxEvtsPerBuf)}

           set z [catch {ReadVMERegister v1725#1.VMEControl} BERR]  ;#  enable BERR to finish a VME block transfer
           if {$BERR < 0} {set BERR 0}
           WriteNetVar NetVar.V1725_BERREnable [expr ($BERR >> 4) & 1]

puts "V1725 Output buffers $BUFF_NUM  @ $Options(V1725_MemBufSize) 32 bit words"
puts "Waveform [expr $SIZE * 10] samples"
puts "16 channels; $NUM events/buffer"

           set osize [expr (($SIZE + 2) * $NUM) + 2]
puts "V1725 buffer size used $osize words out of max $Options(V1725_MemBufSize)"

           if {$osize > $Options(V1725_MemBufSize)} {
              puts "Number of samples or Events per buffer is too big\n\
                 $osize > $Options(V1725_MemBufSize) - this is not permitted\n\
                 correct options and try again"
              return
           }

           set BLT [expr $VME_BUFFER_SIZE / (($osize * 8) + 4)]
           if {$BLT > $Options(V1725_MaxBLT)} {set BLT $Options(V1725_MaxBLT)}

puts "BLT Event Number = $BLT"
           set isize [expr (($osize * 8) + 4) * $BLT]
puts "Data Acq Input buffer size used $isize words out of max $VME_BUFFER_SIZE"

puts "V1725_Configure $BERR $SIZE $BUFF_ORG $NUM"

           WriteVMERegisters v1725#*.VMEControl $BERR     ;# set all modules to same setting
           WriteVMERegisters v1725#*.CustomSize $SIZE     ;# set all modules to same setting
           WriteVMERegisters v1725#*.BufferOrg $BUFF_ORG
           WriteVMERegisters v1725#*.EvtsPerBuf $NUM
           WriteVMERegisters v1725#*.BLTEventNum $BLT     ;# set all modules to same setting
      }

puts "V1725_Configure finished"

}


proc V1725_DAQ-go {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1725#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1725#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1725#$i.AcqControl [expr ($m & 0xfb) + 4]}

   }
}

proc V1725_DAQ-halt {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1725#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1725#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1725#$i.AcqControl [expr ($m & 0xfb)]}

   }
}

proc V1725_DAQ-setup {} {

     V1725_Configure
}

proc V1725_DAQ-flush {} {}

proc V1725_DAQ-reset {} {}

proc define_V1725_config {} {
    variable Options
    variable V1725_Base

#   set maxmodule [inform vmecontrol inquire V1725count]
#   if {[string length $maxmodule] > 2} {set maxmodule 0}

puts "define_V1725_config"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1725#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "17250 0 [format "0x%x" $V1725_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegisterAttr v1725#$i.Output_Buffer} m]
      if {$z!=0} then {
          puts "Attempting to locate register v1725#$i.Output_Buffer"
          error $m
      }

      DefineConfig "17250 $i [string range [first $m] 2 end]"
   }
}

proc create_V1725_spectra {} {
    variable V1725_Base
    variable Options 

puts "create_V1725_spectra"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1725#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

     if {$Options(V1725_Hist_Trace.Enable) == 1} {

        for {set j 0} {$j < 16} {incr j} {   

#    histograms for the data trace

           set spectrumname v1725#$i.Ch[set j]
           set z [catch {CreateSpectrum1D $spectrumname $Options(V1725_Hist_Trace.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group  [expr (($i-1) * 16) + $j + 256  + $V1725_Base] 
              map_spectrum  "$spectrumname $group 0 $Options(V1725_Hist_Trace.offset) $Options(V1725_Hist_Trace.shift)"
           }
        }
     }
   }
}

