

#    experiment run control for CAEN - taken from sys.tcl

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

variable Options

variable SpecBase [file join $env(MIDASBASE) SpecData CAEN]
variable SpecPath


set z [catch {variable Modules $::VMEConfig::ModuleMenu} m]
if {$z != 0} {variable Modules [list CAEN_V1720 CAEN_V1724 CAEN_V1724PHA CAEN_V1725 CAEN_V1725PHA CAEN_V1725PSD CAEN_V1730 CAEN_V1730PHA CAEN_V1730PSD CAEN_V1740 CAEN_V1751 CAEN_V1751PSD CAEN_V1495DPP CAEN_V1495]}

variable Supported_Modules {}
foreach item $Modules {
     if {$item == "SAC"} {lappend Supported_Modules $item; continue}
     if {$item == "PatternGen"} {lappend Supported_Modules $item; continue}
     if {$item == "VData"} {lappend Supported_Modules $item; continue}
     if {$item == "MyRIAD"} {lappend Supported_Modules $item; continue}
     set m [second [split $item _]]
     lappend Supported_Modules $m
}

puts "ConfigBase = $::VMEConfig::ConfigBase"
puts "Supported_Modules = $Supported_Modules"

   proc LoadOptions {} {

          set Project $::DataAcquisition::PROJECT
          set DBRootName [set ::[set Project]::DBRootName]
          set DBNodeBase [set ::[set Project]::DBNodeBase]
          set ::[set Project]::DBNodeName Options

          DBCreateNode $DBRootName/$DBNodeBase/[set ::[set Project]::DBNodeName]

      catch {unset ::Options}

      foreach item [split [DBGetValue Index] &&] {
         if {$item == ""} {continue}
         if {$item == "Index"} {continue}
         if {$item == "Error!"} {continue}
         set vl [DBGetValue $item]
         if {$vl != "Error!"} {set ::Options($item) $vl}
      }

#      InsertLog "Options restored from $DBRootName/$DBNodeBase/[set ::[set Project]::DBNodeName]"

   }

   proc SaveOptions {} {

          set Project $::DataAcquisition::PROJECT
          set DBRootName [set ::[set Project]::DBRootName]
          set DBNodeBase [set ::[set Project]::DBNodeBase]
          set ::[set Project]::DBNodeName Options

          DBCreateNode $DBRootName/$DBNodeBase/[set ::[set Project]::DBNodeName]

      set OptionList ""
      foreach item [array names ::Options] {
         if {$item == ""} {continue}
         if {$item == "Index"} {continue}
         if {$item == "Error!"} {continue}
         if {[string length $OptionList] > 0} {append OptionList "&&"}
         append OptionList "$item"
         DBSetValue "$item" [first [set ::Options($item)]]
      }
      DBSetValue Index $OptionList

#      InsertLog "Options saved to $DBRootName/$DBNodeBase/[set ::[set Project]::DBNodeName]"

   }


proc load_Options {} {LoadOptions}
proc update_Options {} {SaveOptions}


   proc Set_Options {} {
      variable Options
      variable Supported_Modules

      LoadOptions

      if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 512}
      if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 512}
      if {![info exists Options(Stat.shift)]} {set Options(Stat.shift) 6}
      if {![info exists Options(Stat.offset)]} {set Options(Stat.offset) 0}

   if {![info exists Options(VME.settings)]} {set Options(VME.settings) undefined}
   if {![info exists Options(Usepatterngen)]} {set Options(Usepatterngen) 0}

   if {![info exists Options(UseSAC)]} {set Options(UseSAC) 0}
   if {![info exists Options(Use1495)]} {set Options(Use1495) 0}
   if {![info exists Options(Use1495DPP)]} {set Options(Use1495DPP) 0}

      foreach module $Supported_Modules {
         set Options(Include.$module)  1
         set Options(Histogram.$module) 1
      }

      SaveOptions
   }

   proc Dump_Raw_Data {} {
      variable Supported_Modules

#      foreach module $Supported_Modules {append info [Dump_Raw_[set module]_Data]}

    set info "Raw data dump for CAEN VME modules\n"

      set maxmodule 0
      foreach module $Supported_Modules {
         for {set i 1} {$i <= 16} {incr i} {
            set z [catch {ReadVMERegisterAttr [string tolower $module]#$i.VMEStatus} m]
            if {$z!=0} then {break}
            incr maxmodule
         }
      }

    for {set i 1} {$i <= $maxmodule} {incr i} {

       append info "VXXXX#$i\n"

        DataAcqCommand 19 "Getting Raw Data" "1 $i" 2
        set data [ReadNetVar NetVar.EXEC.Response]

        foreach word $data {
           if {[string range $word 0 1] != "0x"} then {set word 0x$word}
           if {$word == 0xaa5555aa} then {
               append info "$word - end of dump\n"
               break
           }
           if {$word == 0x55aaaa55} then {
               append info "$word - end of event\n"
               continue
           }
           append info "$word "
        }

        append info "\n"
     }


       append info "VME\n"

        DataAcqCommand 19 "Getting Raw Data" "5 0" 2
        set data [ReadNetVar NetVar.EXEC.Response]

        foreach word $data {
           if {[string range $word 0 1] != "0x"} then {set word 0x$word}
           if {$word == 0xaa5555aa} then {
               append info "$word - end of dump\n"
               break
           }
           if {$word == 0x55aaaa55} then {
               append info "$word - end of event\n"
               continue
           }
           append info "$word "
        }

        append info "\n"


     return $info

   }

proc doGo {} {

puts "executing CAEN doGo"

     Set_Options; make-UIregisters

     if {[ReadNetVar NetVar.DACQ.Ctxenable] == 1}  {
         DataAcqCommand 3 "connect xfer stream #0" 0 2
     }

     if {[ReadNetVar NetVar.DACQ.Ctx2enable] == 1}  {
         DataAcqCommand 6 "connect xfer stream #1" 0 2
     }

     Reset_Electronics

     DataAcqCommand 2 "Go"

     Go_Electronics

puts "completed CAEN doGo"

}

   proc Stop_Electronics {} {
       variable Options 
       variable Supported_Modules

       if {$Options(Usepatterngen) == 1} {
          WriteVMERegister patterngen#1.Start  0
       }

       if {$Options(UseSAC) != 0} {WriteVMERegister sac#1.STR 0}   ;#  disable acquisiition
       if {$Options(Use1495) != 0} {WriteVMERegister v1495#1.Config.B8 0}     ;#  disable acquisiition
       if {$Options(Use1495DPP) != 0} {WriteVMERegister v1495dpp#1.FT_RUN 0}  ;# disable acquisition

       foreach m $Supported_Modules {[set m]_DAQ-halt}

     after 2000   ;#  allow DA program to flush data 

   }

   proc Go_Electronics {} {
      variable Options 
      variable Supported_Modules

puts "starting Go_Electronics"

puts "Usepatterngen=$Options(Usepatterngen)"
puts "UseSAC=$Options(UseSAC)"
puts "Use1495=$Options(Use1495)"
puts "Use1495DPP=$Options(Use1495DPP)"

       if {$Options(Usepatterngen) == 1} {
           set z [catch {WriteVMERegister patterngen#1.Start  3} m]
       }

       foreach m $Supported_Modules {[set m]_DAQ-go}

       if {$Options(UseSAC) != 0} {WriteVMERegister sac#1.STR 9}
       if {$Options(Use1495DPP) != 0} {WriteVMERegister v1495dpp#1.FT_RUN 1}  ;# enable acquisition
       if {$Options(Use1495) != 0} {WriteVMERegister v1495#1.Config.B8 1}  ;# enable acquisition

puts "Go_Electronics complete"
   }
 
   proc Reset_Electronics {} {    ;#   used by RESET
      variable Supported_Modules

      foreach m $Supported_Modules {[set m]_DAQ-reset}
   }

   proc Setup_Electronics {} {    ;#   used by SETUP
       variable Supported_Modules
       variable Options

set Options(UseSAC) 0
set Options(Use1495) 0
set Options(Use1495DPP) 0

       define_VME_addresses   ;#  setup VME 

       foreach m $Supported_Modules {[set m]_DAQ-setup}

   }

   proc define_VME_addresses {} {
       variable Supported_Modules
       variable Options

#    load hardware configuration into readout software

       ClearConfig
 
       foreach module $Supported_Modules {
          if {$Options(Include.$module) == 1} then {define_[set module]_config}
       }

       DataAcqCommand 4 "print config" 0 2

   }

 proc SpecSave1D {spec base range type title counts opt} {
    variable SpecPath

      if {$opt == 1} {set f3 [file join $SpecPath $spec.dat]} else {set f3 [file join $SpecPath $spec.msf]}
      set z1 [catch {open $f3 w} m1]
      set z2 [catch {file attributes $f3 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f3 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f3 - $m2"
      }

      set f $m1

      if {$opt == 0} {
          fconfigure $f -translation binary -encoding binary
      }

  set ctime [clock format [clock seconds] -format {%d-%b-%Y %H:%M:%S}]
  set mtime [clock format [clock seconds] -format {%d-%b-%Y %H:%M:%S}]

    if {$opt == 0} {

      puts -nonewline $f [binary format i1 412900921]   ;#   magic number
      puts -nonewline $f [binary format i1 1]           ;#   version
      puts -nonewline $f [binary format a32 $spec]      ;#   spectrum name
      puts -nonewline $f [binary format i1 1]           ;#   dimension
      puts -nonewline $f [binary format a20 $ctime]     ;#   creation time/date
      puts -nonewline $f [binary format a20 $mtime]     ;#   modification time/date
      puts -nonewline $f [binary format i1 $base]      ;#   base0

      for {set i 2} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#   baseI
      }

      puts -nonewline $f [binary format i1 $range]     ;#   range0

      for {set i 2} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#   rangeI
      }

      puts -nonewline $f [binary format i1 -1]        ;#   spectrum title offset

      for {set i 2} {$i <=32} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  information pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  annotation pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  calibration pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  efficiency pointers
      }

                                                     ;#  data array 1 descriptor 
      puts -nonewline $f [binary format i1 0]        ;#  array layout
      puts -nonewline $f [binary format i1 $type]        ;#  array type  (32 bit unsigned)
      puts -nonewline $f [binary format i1 0]        ;#  reserved 
      puts -nonewline $f [binary format i1 0]        ;#  reserved
      puts -nonewline $f [binary format i1 0]        ;#  spectrum counts offset

                                                     ;#  data array 2 descriptor 
      puts -nonewline $f [binary format i1 0]       ;#  array layout
      puts -nonewline $f [binary format i1 $type]       ;#  array type 
      puts -nonewline $f [binary format i1 0]        ;#  reserved 
      puts -nonewline $f [binary format i1 0]        ;#  reserved
      puts -nonewline $f [binary format i1 -1]       ;#  spectrum counts offset

#      puts -nonewline $f [binary format i1 436]      ;#  base of string space
#      puts -nonewline $f [binary format i1 20]       ;#  string free space
#      puts -nonewline $f [binary format i1 76]       ;#  top of string space

      puts -nonewline $f [binary format i1 -1]      ;#  base of string space
      puts -nonewline $f [binary format i1 -1]       ;#  string free space
      puts -nonewline $f [binary format i1 -1]       ;#  top of string space

      puts -nonewline $f [binary format i1 512]      ;#  base of counts space
      puts -nonewline $f [binary format i1 $range]       ;#  string free space
      puts -nonewline $f [binary format i1 $range]       ;#  top of string space
 
#      puts -nonewline $f [binary format a20 $title]     ;# string space - title

      for {set i 436} {$i < 512} {incr i} {
         puts -nonewline $f [binary format c1 0]       ;#  unused
      }

      puts -nonewline $f [binary format i* $counts]      ;# counts

    }

    if {$opt == 1} {

      puts -nonewline $f "$spec;"      ;#   spectrum name
      puts -nonewline $f "$ctime;"     ;#   creation time/date
      puts -nonewline $f "$mtime;"     ;#   modification time/date
      puts -nonewline $f "1;"          ;#   dimension
      puts -nonewline $f  "$base;"     ;#   base0
      puts -nonewline $f  "$range;"    ;#   range0
      puts $f "eod"

      puts $f $counts

    }

      close $f
}

    proc HistogramSave {{opt p}} {
       variable SpecBase
       variable SpecPath

        puts "Saving Histograms"

      set f1 $SpecBase
      set z1 [catch {file mkdir $f1} m1]
      set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f1 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f1 - $m2"
      }

#      set f1 [file join $f1 [exec uname -n]]
#      set z1 [catch {file mkdir $f1} m1]
#      set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]

#      if {$z1 != 0} then {
#          puts "Cannot create $f1 - $m1"
#      }
#      if {$z2 != 0} then {
#          puts "error accessing $f1 - $m2"
#      }

      set f2 [file join $f1 [date +%Y%h%d-%H.%M.%S]]
      set z1 [catch {file mkdir $f2} m1]
      set z2 [catch {file attributes $f2 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f2 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f2 - $m2"
      }

       set SpecPath $f2

       set SpecNames [SpectrumNames]
       set NumSpec 0

      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]
 
         set info [SpectrumDetails $SpecName]
         set channels [lindex $info 2]

         set title [SpectrumGetTitle $SpecName]
         set counts [SpectrumRead1D $SpecName 0 $channels]

         set z [catch {SpecSave1D $SpecName 0 $channels 5 $title $counts $opt} m]
         if {$z!=0} then {
             puts "Error attempting to save spectrum $SpecName: $m"
         } else {
             incr NumSpec
         }
      }

        puts "$NumSpec Histograms saved"

    }

    proc HistogramZero {} {

       set SpecNames [SpectrumNames]
       set NumSpec 0

       ZeroSpectrum Stat
       incr NumSpec

      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]

         ZeroSpectrum $SpecName
         incr NumSpec
      }

       puts "$NumSpec spectra zeroed"
    }

    proc TerminateExec {} {

        unmap_spectra
        DataAcqCommand 22 "Terminating DataAcq program" 0 2

    }

    proc LoadExec {} {

       set z [catch {eval exec nice [file join /MIDAS Data_Acq bin Linux64 CAENExecV6] &} m]
       if {$z != 0} {puts "RunControl LoadExec returned z=$z and $m"}

    }


    foreach module $Supported_Modules {

        set f [file join $env(MIDASBASE) TclHttpd Html CAEN RunControl sys.$module.tcl]
        set z [catch {source $f} m]
        if {$z == 0} {puts "RunControl loaded $f"} else {puts "RunControl returned z=$z and $m"}
    }

    make-UIregisters

    Set_Options

    foreach module $Supported_Modules {Set_[set module]_Options}







