

#    experiment run control for PGRIS - taken from sys.tcl

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

variable MaxChans 128
variable MaxDets 8

variable Options

variable Supported_Modules {pgris}


   proc load_Options {} {LoadOptions}
   proc update_Options {} {SaveOptions}

   proc Set_Options {} {
      variable Options
      variable Supported_Modules

      load_Options

   if {![info exists Options(pgris_GroupBase)]} {set Options(pgris_GroupBase) 256}
   if {![info exists Options(Include.pgris)]} {set Options(Include.pgris) 1}

      if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 2000}
      if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 2000}
      if {![info exists Options(Stat.shift)]} {set Options(Stat.shift) 0}
      if {![info exists Options(Stat.offset)]} {set Options(Stat.offset) 1}

   if {![info exists Options(pgris.ASICchannels)]} {set Options(pgris.ASICchannels) 4096}
   if {![info exists Options(pgris.ASICoffset)]} {set Options(pgris.ASICoffset) 1}
   if {![info exists Options(pgris.ASICshift)]} {set Options(pgris.ASICshift) 0}
   
   if {![info exists Options(pgris.ADCchannels)]} {set Options(pgris.ADCchannels) 65536}
   if {![info exists Options(pgris.ADCoffset)]} {set Options(pgris.ADCoffset) 1}
   if {![info exists Options(pgris.ADCshift)]} {set Options(pgris.ADCshift) 0}
   
   if {![info exists Options(pgris.RISEchannels)]} {set Options(pgris.RISEchannels) 64}
   if {![info exists Options(pgris.RISEoffset)]} {set Options(pgris.RISEoffset) 0}
   if {![info exists Options(pgris.RISEshift)]} {set Options(pgris.RISEshift) 0}
   
   if {![info exists Options(pgris.RISEshift)]} {set Options(pgris.MaxHistCells) 5000000}

    if {![info exists Options(DataAcqPgm)]} {set Options(DataAcqPgm) PgrisExecV1}

      update_Options
   }

   proc Dump_Raw_Data {} {
      variable Supported_Modules
   if {![info exists Options(pgris.channels)]} {set Options(pgris.channels) 32768}
   if {![info exists Options(pgris.offset)]} {set Options(pgris.offset) 0}
   if {![info exists Options(pgris.shift)]} {set Options(pgris.shift) 1}

      set info ""

      foreach module $Supported_Modules {append info [Dump_Raw_[set module]_Data]}

      return $info
   }

proc Dump_Raw_pgris_Data {} {

   set info "ASIC data Version 1 : $::DataAcquisition::ACQSERVER\n"
   set c 0
   for {set offset 0} {$offset <= 300} {incr offset 100} {

     DataAcqCommand 19 "Getting Raw Data" "11 $offset" 2
     set data [ReadNetVar NetVar.EXEC.Response]
     set det "Unknown"
     set chan "unknown"
     set timeH 1
     set timeL 2
     foreach word $data {
         set word $word
         if {$word == 0xaa5555aa} then {
             append info "\n$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "Daughter Card:$det , Channel:$chan at [ expr ( $timeH << 19 ) + $timeL ]\n$word - end of event\n"
             continue
         }
         append info "$word "
         set w [expr ($word >> 30) & 3]
         set det [expr ($word >> 26) & 0xf]
         set chan [expr ($word >> 19) & 0x7f]
         set dat [format 0x%4x [expr $word & 0x0000ffff]]
         if { $w == 1 } { set timeH [ expr $word & 0x0007ffff ] }
         if { $w == 2 } { set timeL [ expr $word & 0x0007ffff ] }
         if {$c>10 } { set c 0 ; append info "\n"}
     }
   }

   return "$info"
}
proc doReset {} {

puts "executing doReset for PGRIS"

     Set_Options; make-UIregisters

     Reset_Electronics
	InsertLog "Reset in sys.tcl"

     DataAcqCommand 9 "Reset"

     if {[ReadNetVar NetVar.DACQ.Ctxenable] == 1}  {
         DataAcqCommand 1 "disconnect xfer stream #0" 0 2
     }

     if {[ReadNetVar NetVar.DACQ.Ctx2enable] == 1}  {
         DataAcqCommand 2 "disconnect xfer stream #1" 0 2
     }
}


   proc pgris_DAQ-halt {} {

 puts "pgris_DAQ-halt"
   }

   proc pgris_DAQ-go {} {
	set RDOList $::PGRIS::RDOList

	WritePGRISRegister [ expr 0x20000 + 48 ] 0x1 ; # reset the peripheral state machine
	WritePGRISRegister [ expr 0x20000 + 48 ] 0 ;

	foreach detector { 1 2 3 4 5 6 7 8 9 10 11 } {
 		WritePGRISRegister [ expr 0x20000 + 256 * $detector ] 0xE000 ; # reset the detector state machines and FIFOs
		WritePGRISRegister [ expr 0x20000 + 256 * $detector ] 0 ;
	}

	foreach detector { 1 2 3 4 5 6 7 8 9 10 11 } {
		WritePGRISRegister [ expr 0x20000 + 256 * $detector ] [ lindex $RDOList $detector ] ; # enable this detector for readout if selected
	}
	WritePGRISRegister [ expr 0x20000 + 64 ] 1 ;  # enable the timestamp counter
	set GData [ReadPGRISRegister [expr 0x20000 + 68]]

 	puts "pgris_DAQ-go complete version 2nd March 2015 at 11:30 | Timestamp LSB is $GData"
   }

  proc pgris_DAQ-reset {} {

 puts "pgris_DAQ-reset"
   }

  proc pgris_DAQ-setup {} {

 	puts "pgris_DAQ-setup - all done in GO "
   }

proc HistogramZero {} {

       set SpecNames [SpectrumNames]
       set NumSpec 0

       ZeroSpectrum Stat
       incr NumSpec

      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]

         ZeroSpectrum $SpecName
         incr NumSpec
      }

       puts "$NumSpec spectra zeroed"
    }


proc create_pgris_spectra {} {
   global MaxDcard MaxChans
   variable Options

#   for the data channels
   set Mem_Count 0
   
   set RDOList $::PGRIS::RDOList ;# comes from Pgris.tcl
   
   for {set Dcard 1} {$Dcard < 6 } {incr Dcard} {

      set card [lindex $RDOList $Dcard]
      if {$card <= 0} {continue} ; # daughter card not enabled
      set linecount 10
       puts "Creating spectra for detector $Dcard"
# now add the ASIC channel spectrum         
        set z [catch {CreateSpectrum1D ASIC_$Dcard.ch 200 } m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
           InsertLog "Error attempting to Create spectrum ASIC_$Dcard.ch"
            error $m
         }
         set grp [ expr $Options(pgris_GroupBase) + ( ( $Dcard + 5 ) * 128 ) + 120 ]
         set itm 0 ; # item number in the event format
         set offst 0
         set shft 0

         map_spectrum "ASIC_$Dcard.ch $grp $itm $offst $shft"

    for {set Ch 0} {$Ch < 101} {incr Ch} { ;#Create the simple ASIC ADC spectra

#        set z [catch {CreateSpectrum1D Cell_$card.$Ch.AS $Options(pgris.ASICchannels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
#         if {$z!=0 && $m==0x30004} then {set z 0}
#         if {$z!=0} then {
#           InsertLog "Error attempting to Create spectrum Cell_$card.$Ch.AS"
#            error $m
#         }
#         set grp [ expr $Options(pgris_GroupBase) + ( $card * 128 ) + $Ch ]
#         set itm 0 ; # item number in the event format
#         set offst $Options(pgris.ASICoffset)
#         set shft $Options(pgris.ASICshift)###
#
#         map_spectrum "Cell_$card.$Ch.AS $grp $itm $offst $shft"
#         set Mem_Count [ expr $Mem_Count + $Options(pgris.ASICchannels) ]
         
# now add the external ADC         
        set z [catch {CreateSpectrum1D Cell_$Dcard.$Ch.AD $Options(pgris.ADCchannels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
           InsertLog "Error attempting to Create spectrum Cell_$Dcard.$Ch.AD"
            error $m
         }
         set grp [ expr $Options(pgris_GroupBase) + ( ( $Dcard + 5 ) * 128 ) + $Ch ]
         set itm 0 ; # item number in the event format
         set offst $Options(pgris.ADCoffset)
         set shft $Options(pgris.ADCshift)

         map_spectrum "Cell_$Dcard.$Ch.AD $grp $itm $offst $shft"
         if { $linecount > 0 } { puts "Cell_$Dcard.$Ch.AD $grp $itm $offst $shft" ; set linecount [ expr  $linecount - 1 ] }
         set Mem_Count [ expr $Mem_Count + $Options(pgris.ADCchannels) ]
         
     }
     puts "Memory used for Card $card is $Mem_Count "
   }
}

    make-UIregisters

    Set_Options

