# Note:  This is loaded into the namespace MTEST02V0
# variable started 0

if {![info exists started]} {variable started 0}

if {$started != 0} {
    
    variable JS ""
    variable LogFlag
    
    if {$LogFlag == 1} {InsertLog "Last Updated: [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"}
    
    do_click
}

if {$started == 0} {           ;#  first time load code and define local variables
    set started 1
    
    variable JS ""
    
    proc do_redisplay {} {;}
    
    proc do_test02V0 { params } {
	
	variable msgIden
	
	# set name [lindex [lindex $params 1] 0]
	set sideUnderTest [lindex [lindex $params 1] 1]
	set asicUnderTest [lindex [lindex $params 1] 2]
	set vcall          [lindex [lindex $params 1] 3] ;# [V]
	set vcalh          [lindex [lindex $params 1] 4] ;# [V]
	set stepNumber    [lindex [lindex $params 1] 5] ;# [a.u.]
	set pulseNumber   [lindex [lindex $params 1] 6] ;# [a.u.]
	set pulseWidth    [lindex [lindex $params 1] 7] ;# [10ns]
	set pulseDelay    [lindex [lindex $params 1] 8] ;# [10ns]
	
	do_getState 
	# Start DAQ.
	after 500 set timeout 1
	do_go
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 1 } {return "Daq not started properly.";}
	
	# Loop over individual side.
	foreach itrSide [set ::R3B::[set ::R3B::R3B4K].SIDEList]  {
	    
	    if {$itrSide != $sideUnderTest} {continue;}
	    
	    # Loop over individual asic.
	    foreach itrAsic [set ::R3B::[set ::R3B::R3B4K].ASICList$itrSide]  {
		
		if {$asicUnderTest != "all" && $asicUnderTest != $itrAsic } {continue;}
		
		# Set file name
		set fileName  /MIDAS/SpecData/R3B/TestData/
		append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
		append fileName "Test02V0-"
		append fileName "Side$itrSide-Asic$itrAsic.txt"
		
		# Create file and set permissions.
		set flagOpen [catch {open $fileName w} fileInput]
		set flagPerm [catch {file attributes $fileName -permissions rwxrwxrwx} ]
		if { $flagOpen != 0 } { InsertLog "Cannot open $fileName." }
		if { $flagPerm != 0 } { InsertLog "Error setting permission of $fileName." }
		
		# Enable test pulse injection
		set_asicValidate $itrSide $itrAsic 1
		# Set pulse params
		do_loadPulseParams $pulseWidth $pulseDelay $pulseNumber
		
		# Set pulse amplitude.
		set voltStep [expr 2.0/256] ;# [V]
		set valueVcall [expr round( ($vcall)/$voltStep ) ]
		set registerVcall 17
		WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcall] $valueVcall
		
		set registerVcalh 16
		
		# Loop over individual channels.
		set channelNumber 128
		set channelGroup 8
		for {set itrGroup 0} {$itrGroup < $channelNumber/$channelGroup} {incr itrGroup} {
		    
		    # To run on subset of channel.
		    # if {$itrGroup >=1} { continue;}
		    
		    # Power on channels.
		    for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
		      	set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
		 	set valuePwr [expr 255 - int( pow (2,  $dummyChannel - ( $dummyChannel/8)*8 ) ) ]
		 	set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
		       	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
		    }
		    
		    # Calculate pulse step.
		    set stepSize [expr ($vcalh - $vcall)/$stepNumber ]
		    # Make loop on pulse amplitude.
		    for {set itrStep 0} {$itrStep< $stepNumber} {incr itrStep} {
			
			set valueVcalh [expr round( ($vcall+( ($itrStep+1)*$stepSize) )/$voltStep ) ]
		     	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcalh] $valueVcalh
			# Inject pulses 
		     	do_testPulse $pulseWidth $pulseDelay $pulseNumber
			
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
		     	    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
		     	    set fileName "E.$itrSide.$itrAsic.$dummyChannel"
		     	    # Empty the content of the histogram 
		     	    # Check also that the histogram has been created.
		     	    if { [ZeroSpectrum $fileName] != 0 } { 
		     		return "Histograms E.$itrSide.$itrAsic.* were not created";}
			}
			do_testPulse $pulseWidth $pulseDelay $pulseNumber
			
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set plot  [do_analyseSpectra $itrSide $itrAsic $dummyChannel]
			    set range [lindex $plot 0]
			    set data  [lindex $plot 1]
			    puts $fileInput "$itrSide $itrAsic $dummyChannel $vcall [expr $vcall+ ( ($itrStep+1)*$stepSize)] $range"
			    puts $fileInput $data
			    flush $fileInput
			}
			
		    } ;# end of loop over pulse amplitude.
		    
		    # Power off channel.
		    for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
		    }
		    
		} ;# end of loop over channel. 
		
		close $fileInput
		# Disable test pulse injection
		set_asicValidate $itrSide $itrAsic 0 
		
	    } ;# end of loop over asic.
	} ;# end of loop over side.
	
	# Stop DAQ
	do_getState
	after 500 set timeout 1
	do_stop
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 0 } {return "Daq did not stop correctly";}
	
	InsertLog "Test completed"
    }
    
    proc  do_click {}  {
	variable cgidata
	variable started
	variable LogFlag
	variable ShowLogFlag
	
	set cgidata [ncgi::nvlist]
	
	if {$LogFlag == 1} {InsertLog "cgidata= ([llength $cgidata]) $cgidata"}
	
	if {[llength $cgidata] == 0} then {return}

	if {[llength $cgidata] < 2} then {
	    InsertLog "CGI data is too short: $cgidata"
	} else {
	    if {[lindex $cgidata 0] != "Widget"} then {
		InsertLog "Invalid CGI data: $cgidata"
	    } else {
		
		set w [first [lindex $cgidata 1]]
		
		switch -glob $w {
		    
		    INITIALIZE_ASICS {InsertLog "[do_powerOffFrontEnd]"; InsertLog "[do_enableCalibration]"}
		    INITIALIZE_DAQ {InsertLog "[do_initializeDaq $cgidata]"}
		    RESET    {set started 0}
		    TEST02V0   {InsertLog "[do_test02V0 $cgidata]"}
		    
		    MainLog {}
		    undefined {}
		    default {InsertLog "unexpected CGI data:  $cgidata"}
		}
	    }
        }
        do_redisplay
    }
    
}   ;# end of if {$started == 0}

