# Note:  This is loaded into the namespace  ASIC
# variable started 0

if {![info exists started]} {variable started 0 }

if {$started != 0} {
    variable JS ""
    do_click
}

if {$started == 0} {           ;#  first time load code and define local variables
    set started 1
    
    variable JS ""
    set aa 0
    
    proc do_redisplay {} {
	variable started
	variable aa
	
	InsertLog "Resdisplay";
	InsertLog "started: $started";
	InsertLog "aa: $aa";
	
	set aa [expr $aa+1]
    }
    
    proc do_Test01 { params } {
	
	# set name [lindex [lindex $params 1] 0]
	set side_under_test [lindex [lindex $params 1] 1]
	set asic_under_test [lindex [lindex $params 1] 2]
	set vcall           [lindex [lindex $params 1] 3] ;# [V]
	set vcalh           [lindex [lindex $params 1] 4] ;# [V]
	set pulse_number    [lindex [lindex $params 1] 5] ;# [a.u.]
	set pulse_width     [lindex [lindex $params 1] 6] ;# [10ns]
	set pulse_delay     [lindex [lindex $params 1] 7] ;# [10ns]
	
	# Start DAQ.
	# do_go
	
	# Loop over individual side.
	set side_number 2
        for {set itr_side 0} {$itr_side < $side_number} {incr itr_side} {
	    
	    if {$itr_side !=$side_under_test} {continue;}
	    
	    # Loop over individual asic.
	    set asic_number 16
	    for {set itr_asic 0} {$itr_asic < $asic_number} {incr itr_asic} {
		
		if {$itr_asic !=$asic_under_test} {continue;} 
		## UNCOMMENT TO SCAN MORE THAN 1 CHIP!
		# if {$itr_asic != 12 && $itr_asic != 13 && $itr_asic != 14 && $itr_asic != 15} {continue;}
		# set asic_under_test $itr_asic 
		##
		
	 	# Set file name
		set file_name  /MIDAS/SpecData/R3B/ ;#TestData/
		append file_name [clock format [clock seconds] -format "%Y-%m-%d-"]
		append file_name "Test01-"
		append file_name "Side$itr_side-Chip$itr_asic-Vcall$vcall-Vcalh$vcalh.txt"
		
		# Create file and set permissions.
		set flag_open [catch {open $file_name w} file_input]
		set flag_perm [catch {file attributes $file_name -permissions rwxrwxrwx} ]
		if { $flag_open != 0 } { InsertLog "Cannot open $file_name." }
		if { $flag_perm != 0 } { InsertLog "Error setting permission of $file_name." }
		
		# Set pulse amplitude.
		set volt_step [expr 2.0/256] ;# [V]
		set value_vcalh [expr round( ($vcalh)/$volt_step ) ]
		set value_vcall [expr round( ($vcall)/$volt_step ) ]
		set REGISTER_vcalh 16
		set REGISTER_vcall 17
		WriteR3BASICRegister [expr ($itr_side << 18) + ($itr_asic << 10) + $REGISTER_vcalh] $value_vcalh
		WriteR3BASICRegister [expr ($itr_side << 18) + ($itr_asic << 10) + $REGISTER_vcall] $value_vcall
 		
		# Enable test pulse injection
		set_ASICValidate $itr_side $itr_asic 1
		
		# Loop over individual channels.
		set channel_number 128
		for {set itr_ch 0} {$itr_ch < $channel_number} {incr itr_ch} {
		    
		    if {$itr_ch != 41} { continue;}
		    
		    set value_pwr [expr 255 - pow (2, $itr_ch - ($itr_ch/8)*8 ) ]
		    set REGISTER_pwr [expr 80 + ($itr_ch/8)]
		    WriteR3BASICRegister [expr ($itr_side << 18) + ($itr_asic << 10) + $REGISTER_pwr] $value_pwr
		    
		    # Wait for the channel to switch on properly.
	 	    after 100
		    
		    do_TestPulse $pulse_width $pulse_delay 256
		    
		    set file_name "E.$itr_side.$itr_asic.$itr_ch"
		    # This should zero the histogram.
		    ZeroSpectrum $file_name
		    
		    do_TestPulse $pulse_width $pulse_delay $pulse_number
		    
		    set plot [do_AnalyseSpectra $itr_side $itr_asic $itr_ch]
		    set range [lindex $plot 0]
		    set data  [lindex $plot 1]
		    
		    puts $file_input "$itr_side $itr_asic $itr_ch $vcall $vcalh $range"
		    puts $file_input $data
		    flush $file_input
		    
		    # Power off channel.
		    WriteR3BASICRegister [expr ($itr_side << 18) + ($itr_asic << 10) + $REGISTER_pwr] 255
		} ;# end of loop over channel.
		
		close $file_input
		# Disable test pulse injection
		set_ASICValidate $itr_side $itr_asic 0 
		
	    } ;# end of loop over asic.
	} ;#end of loop over side.
	
	# Stop DAQ
	# do_stop
    }
    
    proc  do_click {}  {
	variable cgidata
	variable started
	variable LogFlag
	variable ShowLogFlag
	
	set cgidata [ncgi::nvlist]
	
	InsertLog "cgidata= ([llength $cgidata]) $cgidata"
	
	if {[llength $cgidata] == 0} then {return}
	
	if {[llength $cgidata] < 2} then {
	    InsertLog "CGI data is too short: $cgidata"
	} else {
	    if {[lindex $cgidata 0] != "Widget"} then {
		InsertLog "Invalid CGI data: $cgidata"
	    } else {
		
		set w [first [lindex $cgidata 1]]
		
		switch -glob $w {
		    
		    INITIALIZE { do_getState }
		    RESET    { set started 0; }
		    TEST01 { do_all }
		    PIPPO  { do_Test01  $cgidata; }
		    
		    default {InsertLog "unexpected CGI data:  $cgidata"}
		}
	    }
        }
	do_redisplay
    }
    
}   ;# end of if {$started == 0}

# InsertLog "[testCommon::do_initializeASICs $cgidata]" ;
