# Note:  This is loaded into the namespace MTEST02
# variable started 0

if {![info exists started]} {variable started 0}

if {$started != 0} {
    
    variable JS ""
    variable LogFlag
    
    if {$LogFlag == 1} {InsertLog "Last Updated: [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"}
    
    do_click
}

if {$started == 0} {           ;#  first time load code and define local variables
    set started 1
    
    variable JS ""
    
    proc do_redisplay {} {;}
    
    proc do_InitializeTest { params } {
	
	# set name [lindex [lindex $params 1] 0]
	set side_under_test   [lindex [lindex $params 1] 1] ;
	set asic_under_test   [lindex [lindex $params 1] 2] ;
	
	# Loop over individual side.
	set side_number 2
	for {set itr_side 0} {$itr_side < $side_number} {incr itr_side} {
	    
	    if { $itr_side!=$side_under_test } {continue;}
	    
	    # Loop over individual asic.
	    set asic_number 16
	    for {set itr_asic 0} {$itr_asic < $asic_number} {incr itr_asic} {
		
		if { $itr_asic!=$asic_under_test } {continue;}
		
		# Loop over individual channels.
		set channel_number 128
		for {set itr_ch 0} {$itr_ch < $channel_number} {incr itr_ch} {
		    
		    # Enable charge injection and power up.
		    set REGISTER_cal [expr 64 + ($itr_ch/8)]
		    set REGISTER_pwr [expr 80 + ($itr_ch/8)]
		    WriteR3BASICRegister [expr ($itr_side << 18) + ($itr_asic << 10) + $REGISTER_pwr] 255
		    WriteR3BASICRegister [expr ($itr_side << 18) + ($itr_asic << 10) + $REGISTER_cal] 255
		} ;# end of loop over channel.
	    } ;# end of loop over asic.
	} ;#end of loop over side.
	
	InsertLog "(SIDE $side_under_test; ASIC $asic_under_test) has been initialized."
    }
    
    proc set_ASICValidate { SIDE ASIC n} {
	
	set vl [ReadR3BASICRegister [expr ($SIDE << 18) + ($ASIC << 10) + 1]]     ;#  REG1
	set vl [expr ($vl & 0xc0) + ($n << 4)]
	WriteR3BASICRegister [expr ($SIDE << 18) + ($ASIC << 10) + 1] $vl
	
    }
    
    proc do_TestPulse {pulse_width pulse_delay pulse_number} {
	
	variable write_ptr1 
	variable write_ptr2
	set write_ptr1 -1
	set write_ptr2 [ReadR3BRegister [expr 0x100 + 7]] 
	
	# Stop any injection of pulses.
	WriteR3BRegister 0x300 0
	# Reset to load values.
	WriteR3BRegister 0x300 4
	
	# Set registers:
	# reg 1 - p_width
	# reg 2 - p_delay 
	# reg 3 - p_number
	WriteR3BRegister [expr 0x300 + 1] $pulse_width
	WriteR3BRegister [expr 0x300 + 2] $pulse_delay
	WriteR3BRegister [expr 0x300 + 3] $pulse_number
	
	# Enable test pulse injection
	# set_ASICValidate 1
	# Start the injection of pulses.
	WriteR3BRegister 0x300 3
	
	variable tm_stamp_msb
	variable tm_stamp_lsb
	variable tm_stamp_1
	variable tm_stamp_2
	
	# Update the variable.
	variable upd
	set upd [ReadR3BRegister 0x100]   ;# update reg 8 => 12
	WriteR3BRegister 0x100 [expr $upd | 0x4]
	WriteR3BRegister 0x100 $upd
	
	# InsertLog "----"
	set write_ptr [ReadR3BRegister [expr 0x100 + 7]]
	set read_ptr [ReadR3BRegister [expr 0x100 + 6]]
	# InsertLog "TestPulse Begin: $write_ptr ,  $read_ptr"
	
	set tm_stamp_msb [ReadR3BRegister [expr 0x100 + 11]]
	set tm_stamp_lsb [ReadR3BRegister [expr 0x100 + 10]]
	set tm_stamp_1 [expr (($tm_stamp_msb & 0xFFFFFFFF) << 32) + ($tm_stamp_lsb & 0xFFFFFFFF)]
	
	set tm_stamp_2 $tm_stamp_1
	
	set counter 0
	set read_ptr -1
	while {$write_ptr2 != $write_ptr1 || $write_ptr2 != $read_ptr || abs($tm_stamp_1 - $tm_stamp_2) < ($pulse_delay * 2) } {
	    set write_ptr1 $write_ptr2
	    set write_ptr2 [ReadR3BRegister [expr 0x100 + 7]] ;
	    set read_ptr [ReadR3BRegister [expr 0x100 + 6]]
	    set counter [expr $counter +1 ] 
	    
	    # Exit for noisy channels.
	    # Very rough approach to the problem.
	    if { $counter>10000 } { InsertLog "Out-out!";return; }
	    # This may be a finer solution: 
	    # if { abs(read_ptr1 - read_ptr2)*4 > ($pulse_number * n)  }
	    # But it is dependent on the burst size and block size ?
	    
	    if {$write_ptr2 == $write_ptr1} {
	    	# Update and read register for time stamp.
	    	set upd [ReadR3BRegister 0x100]   ;# update reg 8 => 12
	    	WriteR3BRegister 0x100 [expr $upd | 0x4]
	    	WriteR3BRegister 0x100 $upd
	    	set tm_stamp_msb [ReadR3BRegister [expr 0x100 + 11]]
	    	set tm_stamp_lsb [ReadR3BRegister [expr 0x100 + 10]]
	    	set tm_stamp_1 [expr (($tm_stamp_msb & 0xFFFFFFFF) << 32) + ($tm_stamp_lsb & 0xFFFFFFFF)]
	    } else {
	    	# Update and read register for time stamp.
	    	set upd [ReadR3BRegister 0x100]   ;# update reg 8 => 12
	    	WriteR3BRegister 0x100 [expr $upd | 0x4]
	    	WriteR3BRegister 0x100 $upd
	    	set tm_stamp_msb [ReadR3BRegister [expr 0x100 + 11]]
	    	set tm_stamp_lsb [ReadR3BRegister [expr 0x100 + 10]]
	    	set tm_stamp_1 [expr (($tm_stamp_msb & 0xFFFFFFFF) << 32) + ($tm_stamp_lsb & 0xFFFFFFFF)]
	    	set tm_stamp_2 $tm_stamp_1
	    }
	    
	}
	
	set write_ptr [ReadR3BRegister [expr 0x100 + 7]]
	set read_ptr [ReadR3BRegister [expr 0x100 + 6]]
	# InsertLog "TestPulse End: $write_ptr ,  $read_ptr"
    }
    
    proc do_AnalyseSpectra { SIDE ASIC CHANNEL } {
	
	set file_name "E.$SIDE.$ASIC.$CHANNEL"
	
	# set SpecNames [SpectrumNames]
	# SpectrumNames is a function if Html/RunControl/sys.tcl 
	# I do not understand how it can be called here.
	# In fact I do not have any include of Html/RunControl/sys.tcl.
	
	set SpecDetails [SpectrumDetails $file_name]
	# SpecDetails has lenght 6.
	# The third element is the range : [0, x-max].
	set range [lindex $SpecDetails 2]
	set data [SpectrumRead1D $file_name 0 $range]
	
	# This should zero the histogram.
	ZeroSpectrum $file_name
	
	return [list $range $data] 
    }
    
    proc do_Test02 { params } {
	
	# Need to get as input file name.
	# Need to check if histogramming box is ticked.
	
	# Check if the DAQ is running.
	if {[do_getstate] != 1} { InsertLog "Daq is NOT going."; return }
	
	# set name [lindex [lindex $params 1] 0]
	set side_under_test [lindex [lindex $params 1] 1]
	set asic_under_test [lindex [lindex $params 1] 2]
	set vcall          [lindex [lindex $params 1] 3] ;# [V]
	set vcalh          [lindex [lindex $params 1] 4] ;# [V]
	set step_number    [lindex [lindex $params 1] 5] ;# [a.u.]
	set pulse_number   [lindex [lindex $params 1] 6] ;# [a.u.]
	set pulse_width    [lindex [lindex $params 1] 7] ;# [10ns]
	set pulse_delay    [lindex [lindex $params 1] 8] ;# [10ns]
	
	# Loop over individual side.
        set side_number 2
        for {set itr_side 0} {$itr_side < $side_number} {incr itr_side} {
	    
	    if {$itr_side !=$side_under_test} {continue;}
	    
	    # Loop over individual asic.
            set asic_number 16
            for {set itr_asic 0} {$itr_asic < $asic_number} {incr itr_asic} {
		
		if {$itr_asic != $asic_under_test} { continue;}
		## UNCOMMENT TO SCAN MORE THAN 1 CHIP!
		#if {$itr_asic != 1 && $itr_asic != 5 && $itr_asic != 9 && $itr_asic != 13} {continue;}
		#set asic_under_test $itr_asic 
		##
		
		# Set file name
		set file_name  /MIDAS/SpecData/R3B/TestData/
		append file_name [clock format [clock seconds] -format "%Y-%m-%d-"]
		append file_name "Test02-"
		append file_name "Side$itr_side-Chip$itr_asic-Vcall$vcall-Vcalh$vcalh-StepN$step_number.txt"
		
		# Create file and set permissions.
		set flag_open [catch {open $file_name w} file_input]
		set flag_perm [catch {file attributes $file_name -permissions rwxrwxrwx} ]
		if { $flag_open != 0 } { InsertLog "Cannot open $file_name." }
		if { $flag_perm != 0 } { InsertLog "Error setting permission of $file_name." }
		# Set pulse amplitude.
		set volt_step [expr 2.0/256] ;# [V]
		set value_vcall [expr round( ($vcall)/$volt_step ) ]
		set REGISTER_vcalh 16
		set REGISTER_vcall 17
		WriteR3BASICRegister [expr ($itr_side << 18) + ($itr_asic << 10) + $REGISTER_vcall] $value_vcall
		
		# Enable test pulse injection
		set_ASICValidate $itr_side $itr_asic 1
		
		# Loop over individual channels.
		set channel_number 128
		for {set itr_ch 0} {$itr_ch < $channel_number} {incr itr_ch} {
		    
		    # if {$itr_ch >= 10} { continue;}
		    
		    # Calculate pulse step.
		    set step_size [expr ($vcalh - $vcall)/$step_number ]
		    # Make loop on pulse amplitude.
		    for {set itr_v 1 } {$itr_v<= $step_number} {incr itr_v} {
			
			set value_vcalh [expr round( ($vcall+($itr_v*$step_size))/$volt_step ) ]
			WriteR3BASICRegister [expr ($itr_side << 18) + ($itr_asic << 10) + $REGISTER_vcalh] $value_vcalh
			
			set value_pwr [expr 255 - pow (2, $itr_ch - ($itr_ch/8)*8 ) ]
			set REGISTER_pwr [expr 80 + ($itr_ch/8)]
			WriteR3BASICRegister [expr ($itr_side << 18) + ($itr_asic << 10) + $REGISTER_pwr] $value_pwr
			# Wait for the channel to switch on.
			after 100
			do_TestPulse $pulse_width $pulse_delay  4
			
			set file_name "E.$itr_side.$itr_asic.$itr_ch"
			# This should zero the histogram.
			ZeroSpectrum $file_name
			
			do_TestPulse $pulse_width $pulse_delay $pulse_number
			
			set plot [do_AnalyseSpectra $itr_side $itr_asic $itr_ch]
			set range [lindex $plot 0]
			set data  [lindex $plot 1]
			
			puts $file_input "$itr_side $itr_asic $itr_ch $vcall [expr $vcall+($itr_v*$step_size)] $range"
			puts $file_input $data
			flush $file_input
			
			# Power off channel.
			WriteR3BASICRegister [expr ($itr_side << 18) + ($itr_asic << 10) + $REGISTER_pwr] 255
		    } ;# end of loop over pulse amplitude.
		    
		} ;# end of loop over channel. 
		close $file_input
		# Disable test pulse injection
		set_ASICValidate $itr_side $itr_asic 0 
		
	    } ;# end of loop over asic.
	} ;# end of loop over side.
	
	InsertLog "Measurement completed."
    }
    
    proc  do_click {}  {
	variable cgidata
	variable started
	variable LogFlag
	variable ShowLogFlag
	
	set cgidata [ncgi::nvlist]
	
	if {$LogFlag == 1} {InsertLog "cgidata= ([llength $cgidata]) $cgidata"}
	
	if {[llength $cgidata] == 0} then {return}

	if {[llength $cgidata] < 2} then {
	    InsertLog "CGI data is too short: $cgidata"
	} else {
	    if {[lindex $cgidata 0] != "Widget"} then {
		InsertLog "Invalid CGI data: $cgidata"
	    } else {
		
		set w [first [lindex $cgidata 1]]
		
		switch -glob $w {
		    
		    INITIALIZE { do_InitializeTest $cgidata;} 
		    RESET    {set started 0}
		    TEST02   { do_Test02 $cgidata }
		    
		    MainLog {}
		    undefined {}
		    default {InsertLog "unexpected CGI data:  $cgidata"}
		}
	    }
        }
        do_redisplay
    }
    
}   ;# end of if {$started == 0}


