
#    MIDAS Access Server  for  ABC module 

package require MIDASDomainHandler       ;#  load default support procedures

package require ABCAccess ;#  load ABC access library


global MIDAS_ABCAccessServer_Verbose
set MIDAS_ABCAccessServer_Verbose 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

global MIDAS_RegisterAccessServer_Verbose
set MIDAS_RegisterAccessServer_Verbose 0

upvar #0 ABCAccessServer_Connected Connected
upvar #0 ABCAccessServer_Xport Xport

set Connected 0

puts "Using ABC Module with Xport at $Xport"


# Use namespaces to isolate your methods
namespace eval urn:ABCAccessServer {

    proc AccessServerVerbose {option} {
        global MIDAS_ABCAccessServer_Verbose
        global MIDAS_RegisterAccessServer_Verbose
        global MIDAS_SOAPDomainHandler_Verbose

        upvar #0 ABCAccessServer_Connected Connected

        set MIDAS_ABCAccessServer_Verbose $option
        set MIDAS_RegisterAccessServer_Verbose $option
        set MIDAS_SOAPDomainHandler_Verbose $option

        return [rpcvar::rpcvar int 0]
    }

    proc Init {} {
        set vl [do_Init]
        return [rpcvar::rpcvar int $vl]
    }

    proc Get {Param Index} {
        set vl [do_Read $Param $Index]
        return [rpcvar::rpcvar int $vl]
    }

    proc Read {Param} {
        set vl [do_Read $Param 32]
        return [rpcvar::rpcvar string $vl]
    }

    proc Set {Param Index Value} {
        set vl [do_Write $Param $Index $Value]
        return [rpcvar::rpcvar int 0]
    }

    proc Write {Param Value} {
        set vl [do_Write $Param 32 $Value]
        return [rpcvar::rpcvar int 0]
    }

    proc Apply {} {
        set vl [do_Apply]
        return [rpcvar::rpcvar int 0]
    }

    proc Load {} {
        set vl [do_Load]
        return [rpcvar::rpcvar int 0]
    }

    proc Status {} {
        set vl [do_Status]
        return [rpcvar::rpcvar string $vl]
    }

    proc Retrieve {} {
        set vl [do_Retrieve]
        return [rpcvar::rpcvar string $vl]
    }

    proc do_Init {} {
        upvar #0 ABCAccessServer_Connected Connected

        set Connected 0

        return 1
    }

    proc do_Write {Param Index Value} {
        upvar #0 MIDAS_ABCAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method ABCAccessServer::Write with parameters $Param $Index $Value"}

#    Check call parameters 

#    Parameter

       switch $Param {
          Relay  -
          Bias   -
          Offset -
          Gain   {}

          default {
              return -code error -errorcode 0x10002 -errorinfo "Parameter specified is invalid"
          }
       }

#    Index    -   0 => 31; 32 = all channels

        if {$Index < 0 || $Index > 32} {
            return -code error -errorcode 0x10003 -errorinfo "Index specified is invalid"
        } else {
            if {$Index == 32} {set AllChans 1} else {set AllChans 0}
        }

#    modify settings required

       switch $Param {
          Relay  {
              if {$AllChans == 0} then {ABCAccess_Relay -i $Index -v $Value} \
                   else {for {set i 0} {$i < 32} {incr i} {ABCAccess_Relay -i $i -v $Value}}
          }
          Bias  {
              if {$AllChans == 0} then {ABCAccess_Bias -i $Index -v $Value} \
                   else {for {set i 0} {$i < 32} {incr i} {ABCAccess_Bias -i $i -v $Value}}
          }
          Offset  {
              if {$AllChans == 0} then {ABCAccess_Offset -i $Index -v $Value} \
                   else {for {set i 0} {$i < 32} {incr i} {ABCAccess_Offset -i $i -v $Value}}
          }
          Gain  {
              if {$AllChans == 0} then {ABCAccess_Gain -i $Index -v $Value} \
                   else {for {set i 0} {$i < 32} {incr i} {ABCAccess_Gain -i $i -v $Value}}
          }
       }

        return     ;# completed 
    }


    proc do_Read {Param Index} {
        upvar #0 MIDAS_ABCAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method ABCAccessServer::Read with parameters $Param $Index"}

#    Check call parameters 

#    Parameter

       switch $Param {
          Relay  -
          Bias   -
          Offset -
          Gain   {}

          default {
              return -code error -errorcode 0x10002 -errorinfo "Parameter specified is invalid"
          }
       }

#    Index    -   0 => 31; 32 = all channels

        if {$Index < 0 || $Index > 32} {
            return -code error -errorcode 0x10003 -errorinfo "Index specified is invalid"
        } else {
            if {$Index == 32} {set AllChans 1} else {set AllChans 0}
        }

#    return settings

       set Values ""

       switch $Param {
          Relay  {
              if {$AllChans == 0} then {append Values [ABCAccess_Relay -i $Index]} \
                   else {for {set i 0} {$i < 32} {incr i} {append Values " [ABCAccess_Relay -i $i]"}}
          }
          Bias  {
              if {$AllChans == 0} then {append Values [ABCAccess_Bias -i $Index]} \
                   else {for {set i 0} {$i < 32} {incr i} {append Values " [ABCAccess_Bias -i $i]"}}
          }
          Offset  {
              if {$AllChans == 0} then {append Values [ABCAccess_Offset -i $Index]} \
                   else {for {set i 0} {$i < 32} {incr i} {append Values " [ABCAccess_Offset -i $i]"}}
          }
          Gain  {
              if {$AllChans == 0} then {append Values [ABCAccess_Gain -i $Index]} \
                   else {for {set i 0} {$i < 32} {incr i} {append Values " [ABCAccess_Gain -i $i]"}}
          }
       }

       return $Values
    }


    proc do_Apply {} {
        upvar #0 MIDAS_ABCAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method ABCAccessServer::Apply"}

#    now start access

        do_Connect

#    do action

       ABCAccess_Apply

        return     ;# completed 
    }

    proc do_Load {} {
        upvar #0 MIDAS_ABCAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method ABCAccessServer::Load"}

#    now start access

        do_Connect

#    do action

       ABCAccess_Load

        return     ;# completed 
    }

    proc do_Status {} {
        upvar #0 MIDAS_ABCAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method ABCAccessServer::Status"}

#    now start access

        do_Connect

#    do action

       set Values [ABCAccess_Status]

        return  $Values   ;# completed 
    }

    proc do_Retrieve {} {
        upvar #0 MIDAS_ABCAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method ABCAccessServer::Retrieve"}

#    now start access

        do_Connect

#    do action

       set Values [ABCAccess_Retrieve]

        return  $Values   ;# completed 
    }


proc do_Connect {} {
        upvar #0 MIDAS_ABCAccessServer_Verbose Verbose
        upvar #0 ABCAccessServer_Xport Xport
        upvar #0 ABCAccessServer_Connected Connected

#    check status of TCP connection to Xport interface for the ABC module

        if {$Connected != 0} {return}

            package require ABCAccess

            ABCAccess_Verbose $Verbose

            set XportHost [first [split $Xport :]]
            set XportPort [second [split $Xport :]]
            ABCAccess_Configure -n $XportHost -p $XportPort  

            ABCAccess_Init

            if {$Verbose} {puts "Access to Xport for ABC module at $XportHost:$XportPort initialised"}

            set Connected 1
}




    # We have to publish the public methods...

     SOAP::export AccessServerVerbose
     SOAP::export Init
     SOAP::export Get
     SOAP::export Set
     SOAP::export Read
     SOAP::export Apply
     SOAP::export Load
     SOAP::export Status
     SOAP::export Retrieve
     SOAP::export Write

     
}   ;#  end of    namespace eval urn:ABCAccessServer

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /ABCAccessServer -namespace urn:ABCAccessServer -uri urn:ABCAccessServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/ABCAccessServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}

package provide ABCAccessServer 1.0

