#    MIDAS Access Server  common procedures 
#    added read class for completeness (SCL, Aug 2010)


set ::MIDAS_RegisterAccessServer_Verbose 0

set ::MIDAS_RegisterAccessServer_Exists 1

    set RegisterBase 0

    proc InquireRegisters {Pattern} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterIndex RegisterIndex
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::InquireRegisters with parameter $Pattern"}
  
        set names [array names RegisterNames -glob $Pattern]

        return [rpcvar::rpcvar string $names]
    }

    proc DeleteRegister {Register} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames
        upvar RegisterBase RegisterBase

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::DeleteRegister with parameter $Register"}

        if {![info exists RegisterNames($Register)]} then {
             return -code error -errorcode 0x10004 -errorinfo "Register name does not exist"
        }

        set Index $RegisterNames($Register)

        if {$Index >= $RegisterBase} {
           upvar R$Index R
           array unset R

           unset RegisterNames($Register)
        }

        return [rpcvar::rpcvar int 0]
    }

    proc DeleteRegisters {Pattern} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames
        upvar RegisterBase RegisterBase

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::DeleteRegisters with parameter $Pattern"}

        foreach Register [array names RegisterNames -glob $Pattern] {
            set Index $RegisterNames($Register)
            if {$Index >= $RegisterBase} {
                upvar R$Index R
                array unset R

                unset RegisterNames($Register)
            }
        }

        return [rpcvar::rpcvar int 0]
    }

    proc WriteRegisterAttributes {Register Attributes} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::WriteRegisterAttributes with parameters $Register $Attributes"}

        if {![info exists RegisterNames($Register)]} then {
             return -code error -errorcode 0x10004 -errorinfo "Register name does not exist"
        }

        do_WriteRegisterAttributes $Register $Attributes

        return [rpcvar::rpcvar int 0]
     }


    proc WriteRegistersAttributes {Pattern Attributes} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::WriteRegistersAttributes with parameters $Pattern $Attributes"}

        foreach Register [array names RegisterNames -glob $Pattern] {

            do_WriteRegisterAttributes $Register $Attributes
        }
  
        return [rpcvar::rpcvar int 0]
    }

    proc ReadRegisterAttributes {Register} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::ReadRegisterAttributes with parameter $Register"}

        if {![info exists RegisterNames($Register)]} then {
             return -code error -errorcode 0x10004 -errorinfo "Register name does not exist"
        }

        return [rpcvar::rpcvar string [do_ReadRegisterAttributes $Register]]
    }

    proc ReadRegistersAttributes {Pattern} {
        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::ReadRegisterAttributes with parameter $Pattern"}

        set Response {}

        foreach Register [array names RegisterNames -glob $Pattern] {

            lappend Response [list $Register [do_ReadRegisterAttributes $Register]]
        }

        return [rpcvar::rpcvar string $Response]
    }

    proc WriteRegister {Register Value} {
        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::WriteRegister with parameters $Register $Value"}

        if {![info exists RegisterNames($Register)]} then {
             return -code error -errorcode 0x10004 -errorinfo "Register name does not exist"
        }

        do_WriteRegister $Register $Value

        return [rpcvar::rpcvar int 0]
    }

    proc WriteRegisters {Pattern Value} {
        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::WriteRegisters with parameters $Pattern $Value"}

        foreach Register [array names RegisterNames -glob $Pattern] {

            do_WriteRegister $Register $Value

        }
  
        return [rpcvar::rpcvar int 0]
    }

    proc InitialiseRegister {Register} {
        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::InitialiseRegister with parameter $Register"}

        if {![info exists RegisterNames($Register)]} then {
             return -code error -errorcode 0x10004 -errorinfo "Register name does not exist"
        }

        set Index $RegisterNames($Register)
        upvar R$Index R

        if {![info exists R(i)]}  then {
             return -code error -errorcode 0x2004 -errorinfo "Register cannot be initialised"
        }

        if {$R(i) == 0xa55a}  then {
             return -code error -errorcode 0x2004 -errorinfo "Register has no initial value"
        }

        do_WriteRegister $Register $R(i)

        return [rpcvar::rpcvar int 0]
    }

    proc InitialiseRegisters {Pattern} {
        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::InitialiseRegisters with parameter $Pattern"}

        foreach Register [array names RegisterNames -glob $Pattern] {

            set Index $RegisterNames($Register)
            upvar R$Index R

            if {![info exists R(i)]}  then {continue}
            if {$R(i) == 0xa55a}  then {continue}

            do_WriteRegister $Register $R(i)
        }

        return [rpcvar::rpcvar int 0]
    }


    proc ReadRegister {Register} {
        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::ReadRegister with parameter $Register"}

        if {![info exists RegisterNames($Register)]} then {
             return -code error -errorcode 0x10004 -errorinfo "Register name does not exist"
        }

        return [rpcvar::rpcvar string [do_ReadRegister $Register]]
    }


    proc ReadRegisters {Pattern} {
        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::ReadRegisters with parameter $Pattern"}

        set Response {}

        foreach Register [array names RegisterNames -glob $Pattern] {

            lappend Response [list $Register [do_ReadRegister $Register]]
        }
 
        return [rpcvar::rpcvar string $Response]
    }


    proc ReadRegisterClass {Register} {
        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::ReadRegisterClass with parameter $Register"}

        if {![info exists RegisterNames($Register)]} then {
             return -code error -errorcode 0x10004 -errorinfo "Register name does not exist"
        }

        set Index $RegisterNames($Register)
        upvar R$Index R
        if {![info exists R(class)]}  then {
             return -code error -errorcode 0x10004 -errorinfo "Register class not registered"
        }
        return [rpcvar::rpcvar string $R(class)]
    }

    proc ReadRegisterClasses {Pattern} {
        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::ReadRegisterClasses with parameter $Pattern"}

        set Response {}

        foreach Register [array names RegisterNames -glob $Pattern] {

            set Index $RegisterNames($Register)
            upvar R$Index R

            if {![info exists R(class)]}  then {continue}

            lappend Response [list $Register $R(class)]
        }
 
        return [rpcvar::rpcvar string $Response]
    }


    variable RegErrorMessages
    set RegErrorMessages([expr 0x10004]) "Register not known"
    set RegErrorMessages([expr 0x20004]) "Register not known"
    set RegErrorMessages([expr 0x20006]) "Register already defined"
    set RegErrorMessages([expr 0x20007]) "Class not known"
    set RegErrorMessages([expr 0x2000a]) "Application detected failure"
    set RegErrorMessages([expr 0x2000e]) "Bad request"
    set RegErrorMessages([expr 0x2004])  "Register not configured for initialisation"

    proc RegisterErrorMessage {Number} {
        global MIDAS_RegisterAccessServer_Verbose
        variable RegErrorMessages
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::RegisterErrorMessage with parameter $Number"}

        if {![info exists RegErrorMessages([expr $Number])]}  then {
             set Response "Error number not recognised"
        } else {
             set Response $RegErrorMessages([expr $Number])
        }
 
        return [rpcvar::rpcvar string $Response]
    }

package provide RegisterAccessServer 1.0
